''*****************************
''* BY JEFF LEDGER/OBC        *
''* TILTSKETCH                *
''*****************************


CON

  _clkmode = xtal1 + pll16x
  _xinfreq =5_000_000               
  'Double buffering is not needed because PSM's GRAM serves as the second buffer
  'So, 12kB ($3000 bytes in hex) is saved. 
  '_stack = ($3000 + $3000 + 100) >> 2   'accomodate display memory and stack
  _stack = (x_tiles*y_tiles*16*4 + 128) >> 2   'accomodate display memory and stack
     'In full screen mode, need to reserve 19,200 bytes (4,800 longs) 
  x_tiles = 20 'PSM can do up to 20    (change to 20 for full screen!)
  y_tiles = 15 'PSM can do up to 15    (change to 15 for full screen!)

  paramcount = 14
  'Since we're not double-buffering, pointing both these display buffer pointers to the same place
  'Shifting the bitmap_base pointer to the end of memory, where display_base also points 
  bitmap_base = ($8000)-x_tiles*y_tiles*16*4-128
  display_base = bitmap_base

  lines = 5
  thickness = 2
  

VAR

  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[x_tiles * y_tiles]
  long  colors[64]

  byte  x[lines]
  byte  y[lines]
  byte  xs[lines]
  byte  ys[lines]

  long  row,col,flag,screensize,rows,cols,color

OBJ

  tv    : "PSM_TV"            'Changed here from "tv" to "PSM_tv"
  gr    : "graphics"
  i2c : "PSM_i2cDriver" 

PUB start | i, j, k, kk, dx, dy, pp, pq, rr, numx, numchr,xx,yy,cl,bt,crayon

  'start tv
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)

  'init colors
  repeat i from 0 to 63
    colors[i] := $00001010 * (i*12/y_tiles+4) & $F + $2B060C02     'slight change here to match

  'init tile screen
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := display_base >> 6 + dy + dx * tv_vc + ((dy & $3F) << 10)


  
  'init bouncing lines
  i := 1001
  j := 123123
  k := 8776434
  repeat i from 0 to lines - 1
    x[i] := ?j // 64
    y[i] := k? // 48
    repeat until xs[i] := k? ~> 29
    repeat until ys[i] := ?j ~> 29

  'start and setup graphics
  gr.start
  gr.setup(x_tiles, y_tiles, x_tiles*16/2, y_tiles*16/2, bitmap_base)

  dira[6]~
  xx:=0
  yy:=0
  crayon:=3
  
  repeat


    gr.colorwidth(crayon,3)
    i:=i2c.GetADC(1)
    j:=i2c.GetADC(2)
    cl:=i2c.GetADC(3)

    bt:=ina[6]
    if bt == 0
      crayon++

    
    if i > 128
      yy++
      gr.plot(xx,yy)

    if i < 128
      yy--
      gr.plot(xx,yy)

    if j < 128
      xx++
      gr.plot(xx,yy)
      
    if j > 128
      xx--
      gr.plot(xx,yy)
  
    'draw small box with text
    gr.colorwidth(1,14)
    gr.box(60,-80,60,16)
    gr.textmode(1,1,6,5)
    gr.colorwidth(2,0)
    gr.text(90,-72,@pchip)

    if cl > 254
     tv.UpdateLcd(true)
      xx:=0
      yy:=0
    
  
    'waitcnt(cnt+clkfreq/50)  'Need a brief delay here to let graphics complete last operation... 
    tv.UpdateLcd(false) 

    'increment counter that makes everything change
    k++

PUB out(c) | i, k

'' Output a character
''
''     $00 = clear screen
''     $01 = home
''     $08 = backspace
''     $09 = tab (8 spaces per)
''     $0A = set X position (X follows)
''     $0B = set Y position (Y follows)
''     $0C = set color (color follows)
''     $0D = return
''  others = printable characters

  case flag
    $00: case c
           $00: wordfill(@screen, $220, screensize)
                col := row := 0
           $01: col := row := 0
           $08: if col
                  col--
           $09: repeat
                  print(" ")
                while col & 7
           $0A..$0C: flag := c
                     return
           $0D: newline
           other: print(c)
    $0A: col := c // cols
    $0B: row := c // rows
    $0C: color := c & 7
  flag := 0
  
PRI print(c)|k,i
      k := color << 1 + c & 1
      i := k << 10 + $200 + c & $FE
      'screen[row * cols + col] := i
      'screen[(row + 1) * cols + col] := i | 1
      if ++col == cols

PRI newline : i

  col := 0
  if (row += 2) == rows
    row -= 2
    'scroll lines
    repeat i from 0 to rows-3
'      wordmove(@screen[i*cols], @screen[(i+2)*cols], cols)
    'clear new line
'    wordfill(@screen[(rows-2)*cols], $200, cols<<1)

DAT

tvparams                long    0               'status
                        long    1               'enable
                        long    %001_0101       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    0               'broadcast
                        long    0               'auralcog

vecdef                  word    $4000+$2000/3*0         'triangle
                        word    50
                        word    $8000+$2000/3*1+1
                        word    50
                        word    $8000+$2000/3*2-1
                        word    50
                        word    $8000+$2000/3*0
                        word    50
                        word    0

vecdef2                 word    $4000+$2000/12*0        'star
                        word    50
                        word    $8000+$2000/12*1
                        word    20
                        word    $8000+$2000/12*2
                        word    50
                        word    $8000+$2000/12*3
                        word    20
                        word    $8000+$2000/12*4
                        word    50
                        word    $8000+$2000/12*5
                        word    20
                        word    $8000+$2000/12*6
                        word    50
                        word    $8000+$2000/12*7
                        word    20
                        word    $8000+$2000/12*8
                        word    50
                        word    $8000+$2000/12*9
                        word    20
                        word    $8000+$2000/12*10
                        word    50
                        word    $8000+$2000/12*11
                        word    20
                        word    $8000+$2000/12*0
                        word    50
                        word    0

pixdef                  word                            'crosshair
                        byte    2,7,3,3
                        word    %%00333000,%%00000000
                        word    %%03020300,%%00000000
                        word    %%30020030,%%00000000
                        word    %%32222230,%%00000000
                        word    %%30020030,%%02000000
                        word    %%03020300,%%22200000
                        word    %%00333000,%%02000000

pixdef2                 word                            'dog
                        byte    1,4,0,3
                        word    %%20000022
                        word    %%02222222
                        word    %%02222200
                        word    %%02000200

pchip                   byte    "TiltSketch",0           'text